@@ -109,7 +109,8 @@ typedef enum {
   GRN_TOO_SMALL_OFFSET = -67,
   GRN_TOO_LARGE_OFFSET = -68,
   GRN_TOO_SMALL_LIMIT = -69,
-  GRN_CAS_ERROR = -70
+  GRN_CAS_ERROR = -70,
+  GRN_UNSUPPORTED_COMMAND_VERSION = -71
 } grn_rc;
 
 GRN_API grn_rc grn_init(void);
@@ -125,6 +126,16 @@ typedef enum {
   GRN_ENC_KOI8R
 } grn_encoding;
 
+typedef enum {
+  GRN_COMMAND_VERSION_DEFAULT = 0,
+  GRN_COMMAND_VERSION_1,
+  GRN_COMMAND_VERSION_2
+} grn_command_version;
+
+#define GRN_COMMAND_VERSION_MIN GRN_COMMAND_VERSION_1
+#define GRN_COMMAND_VERSION_STABLE GRN_COMMAND_VERSION_1
+#define GRN_COMMAND_VERSION_MAX GRN_COMMAND_VERSION_2
+
 typedef enum {
   GRN_LOG_NONE = 0,
   GRN_LOG_EMERG,
@@ -242,6 +253,75 @@ GRN_API grn_rc grn_set_default_encoding(grn_encoding encoding);
 GRN_API const char *grn_get_version(void);
 GRN_API const char *grn_get_package(void);
 
+/**
+ * grn_get_default_command_version:
+ *
+ * デフォルトのcommand_versionを返します。
+ **/
+GRN_API grn_command_version grn_get_default_command_version(void);
+
+/**
+ * grn_set_default_command_version:
+ * @version: 変更後のデフォルトのcommand_versionを指定します。
+ *
+ * デフォルトのcommand_versionを変更します。
+ **/
+GRN_API grn_rc grn_set_default_command_version(grn_command_version version);
+
+/**
+ * grn_ctx_get_command_version:
+ *
+ * command_versionを返します。
+ **/
+GRN_API grn_command_version grn_ctx_get_command_version(grn_ctx *ctx);
+
+/**
+ * grn_ctx_set_command_version:
+ * @version: 変更後のcommand_versionを指定します。
+ *
+ * command_versionを変更します。
+ **/
+GRN_API grn_rc grn_ctx_set_command_version(grn_ctx *ctx, grn_command_version version);
+
+/**
+ * grn_ctx_get_match_escalation_threshold:
+ *
+ * 検索の挙動をエスカレーションする閾値を返します。エスカレー
+ * ションの詳細は検索の仕様に関するドキュメントを参照してく
+ * ださい。
+ **/
+GRN_API long long int grn_ctx_get_match_escalation_threshold(grn_ctx *ctx);
+
+/**
+ * grn_ctx_set_match_escalation_threshold:
+ * @threshold: 変更後の検索の挙動をエスカレーションする閾値を指定します。
+ *
+ * 検索の挙動をエスカレーションする閾値を変更します。エスカレー
+ * ションの詳細は検索の仕様に関するドキュメントを参照してくだ
+ * さい。
+ **/
+GRN_API grn_rc grn_ctx_set_match_escalation_threshold(grn_ctx *ctx, long long int threshold);
+
+/**
+ * grn_get_default_match_escalation_threshold:
+ *
+ * デフォルトの検索の挙動をエスカレーションする閾値を返します。
+ * エスカレーションの詳細は検索の仕様に関するドキュメントを参
+ * 照してください。
+ **/
+GRN_API long long int grn_get_default_match_escalation_threshold(void);
+
+/**
+ * grn_set_default_match_escalation_threshold:
+ * @threshold: 変更後のデフォルトの検索の挙動をエスカレーションする閾値を指定します。
+ *
+ * デフォルトの検索の挙動をエスカレーションする閾値を変更しま
+ * す。エスカレーションの詳細は詳細は検索の仕様に関するドキュ
+ * メントを参照してください。
+ **/
+GRN_API grn_rc grn_set_default_match_escalation_threshold(long long int threshold);
+
+
 /* obj */
 
 typedef unsigned short int grn_obj_flags;
@@ -713,13 +793,11 @@ typedef grn_obj grn_table_cursor;
  *         minは無視される。
  *
  *         maxとmax_sizeが指定され、かつ、テーブルのkeyが固定長型の場合、
- *         maxと値が近い順番にレコードを取り出す。
+ *         maxとPAT木上で近い位置にあるノードから順番にレコードを取り出す。
  *         ただし、keyのパトリシア木で、min_sizeバイト未満のビットに対する
  *         ノードで、maxと異なった方向にあるノードに対応するレコードについては
  *         取り出さない。
- *         「keyが近い」ことの定義は、keyの型によって異なる。
- *         (GeoPoint型では、地理的に近いものほど近いものとし、
- *          数値型では、数値が近いものほど近いものとする)
+ *         PAT木上で位置が近いこととkeyの値が近いことは同一ではない。
  *         この場合、maxで与えられるポインタが指す値は、
  *         対象テーブルのkeyサイズと同じか超える幅である必要がある。
  *         minは無視される。
@@ -815,6 +893,42 @@ GRN_API grn_rc grn_table_cursor_delete(grn_ctx *ctx, grn_table_cursor *tc);
  **/
 GRN_API grn_obj *grn_table_cursor_table(grn_ctx *ctx, grn_table_cursor *tc);
 
+typedef struct {
+  grn_id rid;
+  grn_id sid;
+  unsigned int pos;
+  unsigned int tf;
+  unsigned int weight;
+  unsigned int rest;
+} grn_posting;
+
+/**
+ * grn_index_cursor_open
+ * @tc: 対象cursor
+ * @index: 対象cursor
+ * @rid_min: 出力するレコードidの下限
+ * @rid_max: 出力するレコードidの上限
+ *
+ * grn_table_cursorから取得できるそれぞれのレコードについて、
+ * GRN_OBJ_COLUMN_INDEX型のカラムの値を順番に取り出すためのカーソルを生成して返す。
+ * rid_min,rid_maxを指定して取得するレコードidの値を制限することができる。
+ * 戻り値であるgrn_index_cursorはgrn_obj_closeを使って解放する。
+ **/
+GRN_API grn_obj *grn_index_cursor_open(grn_ctx *ctx, grn_table_cursor *tc, grn_obj *index,
+                                       grn_id rid_min, grn_id rid_max, int flags);
+
+/**
+ * grn_index_cursor_next
+ * @ic: 対象cursor
+ * @tid: テーブルレコードID
+ *
+ * cursorの範囲内のインデックスの値を順番に取り出す。
+ * tidにNULL以外を指定した場合は、
+ * index_cursorを作成するときに指定したtable_cursorの現在の対象レコードのidを返す。
+ * 戻り値であるgrn_posting構造体は解放する必要はない。
+ **/
+GRN_API grn_posting *grn_index_cursor_next(grn_ctx *ctx, grn_obj *ic, grn_id *tid);
+
 #define GRN_TABLE_EACH(ctx,table,head,tail,id,key,key_size,value,block) do {\
   (ctx)->errlvl = GRN_LOG_NOTICE;\
   (ctx)->rc = GRN_SUCCESS;\
