@@ -62,13 +62,23 @@ int grn_uyield_count = 0;
 grn_rc
 grn_timeval_now(grn_ctx *ctx, grn_timeval *tv)
 {
+#ifdef HAVE_CLOCK_GETTIME
+  struct timespec t;
+  if (clock_gettime(CLOCK_REALTIME, &t)) {
+    SERR("clock_gettime");
+  } else {
+    tv->tv_sec = (int32_t) t.tv_sec;
+    tv->tv_nsec = t.tv_nsec;
+  }
+  return ctx->rc;
+#else /* HAVE_CLOCK_GETTIME */
 #ifdef WIN32
   time_t t;
   struct _timeb tb;
   time(&t);
   _ftime(&tb);
   tv->tv_sec = (int32_t) t;
-  tv->tv_usec = tb.millitm * 1000;
+  tv->tv_nsec = tb.millitm * (GRN_TIME_NSEC_PER_SEC / 1000);
   return GRN_SUCCESS;
 #else /* WIN32 */
   struct timeval t;
@@ -76,10 +86,11 @@ grn_timeval_now(grn_ctx *ctx, grn_timeval *tv)
     SERR("gettimeofday");
   } else {
     tv->tv_sec = (int32_t) t.tv_sec;
-    tv->tv_usec = t.tv_usec;
+    tv->tv_nsec = GRN_TIME_USEC_TO_NSEC(t.tv_usec);
   }
   return ctx->rc;
 #endif /* WIN32 */
+#endif /* HAVE_CLOCK_GETTIME */
 }
 
 void
@@ -87,7 +98,8 @@ grn_time_now(grn_ctx *ctx, grn_obj *obj)
 {
   grn_timeval tv;
   grn_timeval_now(ctx, &tv);
-  GRN_TIME_SET(ctx, obj, GRN_TIME_PACK(tv.tv_sec, tv.tv_usec));
+  GRN_TIME_SET(ctx, obj, GRN_TIME_PACK(tv.tv_sec,
+                                       GRN_TIME_NSEC_TO_USEC(tv.tv_nsec)));
 }
 
 grn_rc
@@ -105,7 +117,8 @@ grn_timeval2str(grn_ctx *ctx, grn_timeval *tv, char *buf)
   if (!ltm) { SERR("localtime"); }
   snprintf(buf, GRN_TIMEVAL_STR_SIZE - 1, GRN_TIMEVAL_STR_FORMAT,
            ltm->tm_year + 1900, ltm->tm_mon + 1, ltm->tm_mday,
-           ltm->tm_hour, ltm->tm_min, ltm->tm_sec, (int) tv->tv_usec);
+           ltm->tm_hour, ltm->tm_min, ltm->tm_sec,
+           (int)(tv->tv_nsec / GRN_TIME_NSEC_PER_SEC * GRN_TIME_USEC_PER_SEC));
   buf[GRN_TIMEVAL_STR_SIZE - 1] = '\0';
   return ctx->rc;
 }
@@ -156,8 +169,8 @@ grn_str2timeval(const char *str, uint32_t str_len, grn_timeval *tv)
     uv *= 10;
     r2++;
   }
-  if (uv >= 1000000) { return GRN_INVALID_ARGUMENT; }
-  tv->tv_usec = uv;
+  if (uv >= GRN_TIME_USEC_PER_SEC) { return GRN_INVALID_ARGUMENT; }
+  tv->tv_nsec = GRN_TIME_USEC_TO_NSEC(uv);
   return GRN_SUCCESS;
 }
 
@@ -241,6 +254,9 @@ grn_ctx_impl_init(grn_ctx *ctx)
   }
 #ifdef USE_DYNAMIC_MALLOC_CHANGE
   grn_ctx_impl_init_malloc(ctx);
+#endif
+#ifdef ENABLE_MEMORY_DEBUG
+  ctx->impl->alloc_info = NULL;
 #endif
   ctx->impl->encoding = ctx->encoding;
   ctx->impl->lifoseg = -1;
@@ -263,6 +279,20 @@ grn_ctx_impl_init(grn_ctx *ctx)
   GRN_TEXT_INIT(&ctx->impl->names, GRN_OBJ_VECTOR);
   GRN_UINT32_INIT(&ctx->impl->levels, GRN_OBJ_VECTOR);
 
+  if (ctx == &grn_gctx) {
+    ctx->impl->command_version = GRN_COMMAND_VERSION_STABLE;
+  } else {
+    ctx->impl->command_version = grn_get_default_command_version();
+  }
+
+  if (ctx == &grn_gctx) {
+    ctx->impl->match_escalation_threshold =
+      GROONGA_DEFAULT_MATCH_ESCALATION_THRESHOLD;
+  } else {
+    ctx->impl->match_escalation_threshold =
+      grn_get_default_match_escalation_threshold();
+  }
+
   ctx->impl->phs = NIL;
   ctx->impl->code = NIL;
   ctx->impl->dump = NIL;
@@ -291,7 +321,7 @@ grn_ctx_impl_init(grn_ctx *ctx)
   ctx->impl->output = NULL /* grn_ctx_concat_func */;
   ctx->impl->data.ptr = NULL;
   ctx->impl->tv.tv_sec = 0;
-  ctx->impl->tv.tv_usec = 0;
+  ctx->impl->tv.tv_nsec = 0;
   GRN_TEXT_INIT(&ctx->impl->subbuf, 0);
   ctx->impl->edge = NULL;
   grn_loader_init(&ctx->impl->loader);
@@ -462,12 +492,12 @@ grn_ctx_fin(grn_ctx *ctx)
         }
       }
     }
+    CRITICAL_SECTION_FIN(ctx->impl->lock);
     {
       grn_io_mapinfo mi;
       mi.map = (void *)ctx->impl;
       grn_io_anon_unmap(ctx, &mi, IMPL_SIZE);
     }
-    CRITICAL_SECTION_FIN(ctx->impl->lock);
   }
   ctx->stat = GRN_CTX_FIN;
   return rc;
@@ -691,6 +721,30 @@ grn_set_default_encoding(grn_encoding encoding)
   }
 }
 
+grn_command_version
+grn_get_default_command_version(void)
+{
+  return grn_ctx_get_command_version(&grn_gctx);
+}
+
+grn_rc
+grn_set_default_command_version(grn_command_version version)
+{
+  return grn_ctx_set_command_version(&grn_gctx, version);
+}
+
+long long int
+grn_get_default_match_escalation_threshold(void)
+{
+  return grn_ctx_get_match_escalation_threshold(&grn_gctx);
+}
+
+grn_rc
+grn_set_default_match_escalation_threshold(long long int threshold)
+{
+  return grn_ctx_set_match_escalation_threshold(&grn_gctx, threshold);
+}
+
 static int alloc_count = 0;
 
 grn_rc
@@ -745,6 +799,51 @@ grn_ctx_close(grn_ctx *ctx)
   return rc;
 }
 
+grn_command_version
+grn_ctx_get_command_version(grn_ctx *ctx)
+{
+  if (ctx->impl) {
+    return ctx->impl->command_version;
+  } else {
+    return GRN_COMMAND_VERSION_STABLE;
+  }
+}
+
+grn_rc
+grn_ctx_set_command_version(grn_ctx *ctx, grn_command_version version)
+{
+  switch (version) {
+  case GRN_COMMAND_VERSION_DEFAULT :
+    ctx->impl->command_version = GRN_COMMAND_VERSION_STABLE;
+    return GRN_SUCCESS;
+  default :
+    if (GRN_COMMAND_VERSION_MIN <= version &&
+        version <= GRN_COMMAND_VERSION_MAX) {
+      ctx->impl->command_version = version;
+      return GRN_SUCCESS;
+    } else {
+      return GRN_UNSUPPORTED_COMMAND_VERSION;
+    }
+  }
+}
+
+long long int
+grn_ctx_get_match_escalation_threshold(grn_ctx *ctx)
+{
+  if (ctx->impl) {
+    return ctx->impl->match_escalation_threshold;
+  } else {
+    return GROONGA_DEFAULT_MATCH_ESCALATION_THRESHOLD;
+  }
+}
+
+grn_rc
+grn_ctx_set_match_escalation_threshold(grn_ctx *ctx, long long int threshold)
+{
+  ctx->impl->match_escalation_threshold = threshold;
+  return GRN_SUCCESS;
+}
+
 grn_content_type
 grn_get_ctype(grn_obj *var)
 {
@@ -849,10 +948,34 @@ grn_str_get_mime_type(grn_ctx *ctx, const char *p, const char *pe,
   }
 }
 
+static void
+get_command_version(grn_ctx *ctx, const char *p, const char *pe)
+{
+  grn_command_version version;
+  const char *rest;
+
+  version = grn_atoui(p, pe, &rest);
+  if (pe == rest) {
+    grn_rc rc;
+    rc = grn_ctx_set_command_version(ctx, version);
+    if (rc == GRN_UNSUPPORTED_COMMAND_VERSION) {
+      ERR(rc,
+          "unsupported command version is specified: %d: "
+          "stable command version: %d: "
+          "available command versions: %d-%d",
+          version,
+          GRN_COMMAND_VERSION_STABLE,
+          GRN_COMMAND_VERSION_MIN, GRN_COMMAND_VERSION_MAX);
+    }
+  }
+}
+
 #define INDEX_HTML "index.html"
 #define OUTPUT_TYPE "output_type"
+#define COMMAND_VERSION "command_version"
 #define EXPR_MISSING "expr_missing"
 #define OUTPUT_TYPE_LEN (sizeof(OUTPUT_TYPE) - 1)
+#define COMMAND_VERSION_LEN (sizeof(COMMAND_VERSION) - 1)
 
 static inline int
 command_proc_p(grn_obj *expr)
@@ -885,6 +1008,12 @@ grn_ctx_qe_exec_uri(grn_ctx *ctx, const char *path, uint32_t path_len)
         p = grn_text_cgidec(ctx, &buf, p, e, '&');
         v = GRN_TEXT_VALUE(&buf);
         get_content_mime_type(ctx, v, GRN_BULK_CURR(&buf));
+      } else if (l == COMMAND_VERSION_LEN &&
+                 !memcmp(v, COMMAND_VERSION, COMMAND_VERSION_LEN)) {
+        GRN_BULK_REWIND(&buf);
+        p = grn_text_cgidec(ctx, &buf, p, e, '&');
+        get_command_version(ctx, GRN_TEXT_VALUE(&buf), GRN_BULK_CURR(&buf));
+        if (ctx->rc) { goto exit; }
       } else {
         if (!(val = grn_expr_get_or_add_var(ctx, expr, v, l))) {
           val = &buf;
@@ -904,6 +1033,7 @@ grn_ctx_qe_exec_uri(grn_ctx *ctx, const char *path, uint32_t path_len)
     ctx->impl->curr_expr = expr;
     grn_expr_exec(ctx, expr, 0);
   }
+exit :
   GRN_OBJ_FIN(ctx, &buf);
   return expr;
 }
@@ -935,6 +1065,12 @@ grn_ctx_qe_exec(grn_ctx *ctx, const char *str, uint32_t str_len)
           p = grn_text_unesc_tok(ctx, &buf, p, e, &tok_type);
           v = GRN_TEXT_VALUE(&buf);
           get_content_mime_type(ctx, v, GRN_BULK_CURR(&buf));
+        } else if (l == COMMAND_VERSION_LEN &&
+                   !memcmp(v, COMMAND_VERSION, COMMAND_VERSION_LEN)) {
+          GRN_BULK_REWIND(&buf);
+          p = grn_text_unesc_tok(ctx, &buf, p, e, &tok_type);
+          get_command_version(ctx, GRN_TEXT_VALUE(&buf), GRN_BULK_CURR(&buf));
+          if (ctx->rc) { goto exit; }
         } else if (expr && (val = grn_expr_get_or_add_var(ctx, expr, v, l))) {
           grn_obj_reinit(ctx, val, GRN_DB_TEXT, 0);
           p = grn_text_unesc_tok(ctx, val, p, e, &tok_type);
@@ -966,6 +1102,7 @@ grn_ctx_qe_exec(grn_ctx *ctx, const char *str, uint32_t str_len)
           GRN_TEXT_LEN(&buf), GRN_TEXT_VALUE(&buf));
     }
   }
+exit :
   GRN_OBJ_FIN(ctx, &buf);
   return expr;
 }
@@ -1040,12 +1177,7 @@ grn_ctx_send(grn_ctx *ctx, const char *str, unsigned int str_len, int flags)
       }
       if (expr) { grn_expr_clear_vars(ctx, expr); }
       if (!ctx->impl->qe_next) {
-        uint64_t et;
-        grn_timeval tv;
-        grn_timeval_now(ctx, &tv);
-        et = (tv.tv_sec - ctx->impl->tv.tv_sec) * GRN_TIME_USEC_PER_SEC
-          + (tv.tv_usec - ctx->impl->tv.tv_usec);
-        GRN_LOG(ctx, GRN_LOG_NONE, "%08x|<%012zu rc=%d", (intptr_t)ctx, et, ctx->rc);
+        LAP("<", "rc=%d", ctx->rc);
       }
       goto exit;
     }
@@ -1787,6 +1919,109 @@ grn_strdup(grn_ctx *ctx, const char *string, const char* file, int line, const c
 }
 #endif
 
+#ifdef ENABLE_MEMORY_DEBUG
+inline static void
+grn_alloc_info_set_backtrace(char *buffer, size_t size)
+{
+#  define N_TRACE_LEVEL 100
+  static void *trace[N_TRACE_LEVEL];
+  char **symbols;
+  int i, n, rest;
+
+  rest = size;
+  n = backtrace(trace, N_TRACE_LEVEL);
+  symbols = backtrace_symbols(trace, n);
+  if (symbols) {
+    for (i = 0; i < n; i++) {
+      int symbol_length;
+
+      symbol_length = strlen(symbols[i]);
+      if (symbol_length + 2 > rest) {
+        break;
+      }
+      memcpy(buffer, symbols[i], symbol_length);
+      buffer += symbol_length;
+      rest -= symbol_length;
+      buffer[0] = '\n';
+      buffer++;
+      rest--;
+      buffer[0] = '\0';
+      rest--;
+    }
+    free(symbols);
+  }
+#  undef N_TRACE_LEVEL
+}
+
+inline static void
+grn_alloc_info_add(void *address)
+{
+  grn_ctx *ctx;
+  grn_alloc_info *new_alloc_info;
+
+  ctx = &grn_gctx;
+  if (!ctx->impl) { return; }
+
+  new_alloc_info = malloc(sizeof(grn_alloc_info));
+  new_alloc_info->address = address;
+  new_alloc_info->freed = GRN_FALSE;
+  new_alloc_info->alloc_backtrace[0] = '\0';
+  grn_alloc_info_set_backtrace(new_alloc_info->alloc_backtrace,
+                               sizeof(new_alloc_info->alloc_backtrace));
+  new_alloc_info->next = ctx->impl->alloc_info;
+  ctx->impl->alloc_info = new_alloc_info;
+}
+
+inline static void
+grn_alloc_info_dump(grn_ctx *ctx)
+{
+  int i = 0;
+  grn_alloc_info *alloc_info;
+
+  if (!ctx) { return; }
+  if (!ctx->impl) { return; }
+
+  alloc_info = ctx->impl->alloc_info;
+  for (; alloc_info; alloc_info = alloc_info->next) {
+    printf("address[%d]: %p\n", i, alloc_info->address);
+    i++;
+  }
+}
+
+inline static void
+grn_alloc_info_check(void *address)
+{
+  grn_ctx *ctx;
+  grn_alloc_info *alloc_info;
+
+  ctx = &grn_gctx;
+  if (!ctx->impl) { return; }
+  /* grn_alloc_info_dump(ctx); */
+
+  alloc_info = ctx->impl->alloc_info;
+  for (; alloc_info; alloc_info = alloc_info->next) {
+    if (alloc_info->address == address) {
+      if (alloc_info->freed) {
+        GRN_LOG(ctx, GRN_LOG_WARNING,
+                "double free: (%p):\nalloc backtrace:\n%sfree backtrace:\n%s",
+                alloc_info->address,
+                alloc_info->alloc_backtrace,
+                alloc_info->free_backtrace);
+      } else {
+        alloc_info->freed = GRN_TRUE;
+        alloc_info->free_backtrace[0] = '\0';
+        grn_alloc_info_set_backtrace(alloc_info->free_backtrace,
+                                     sizeof(alloc_info->free_backtrace));
+      }
+      return;
+    }
+  }
+}
+#else /* ENABLE_MEMORY_DEBUG */
+#  define grn_alloc_info_add(address)
+#  define grn_alloc_info_check(address)
+#endif /* ENABLE_MEMORY_DEBUG */
+
 void *
 grn_malloc_default(grn_ctx *ctx, size_t size, const char* file, int line, const char *func)
 {
@@ -1795,11 +2030,13 @@ grn_malloc_default(grn_ctx *ctx, size_t size, const char* file, int line, const
     void *res = malloc(size);
     if (res) {
       GRN_ADD_ALLOC_COUNT(1);
+      grn_alloc_info_add(res);
     } else {
       if (!(res = malloc(size))) {
         MERR("malloc fail (%d)=%p (%s:%d) <%d>", size, res, file, line, alloc_count);
       } else {
         GRN_ADD_ALLOC_COUNT(1);
+        grn_alloc_info_add(res);
       }
     }
     return res;
@@ -1814,11 +2051,13 @@ grn_calloc_default(grn_ctx *ctx, size_t size, const char* file, int line, const
     void *res = calloc(size, 1);
     if (res) {
       GRN_ADD_ALLOC_COUNT(1);
+      grn_alloc_info_add(res);
     } else {
       if (!(res = calloc(size, 1))) {
         MERR("calloc fail (%d)=%p (%s:%d) <%d>", size, res, file, line, alloc_count);
       } else {
         GRN_ADD_ALLOC_COUNT(1);
+        grn_alloc_info_add(res);
       }
     }
     return res;
@@ -1829,6 +2068,7 @@ void
 grn_free_default(grn_ctx *ctx, void *ptr, const char* file, int line, const char *func)
 {
   if (!ctx) { return; }
+  grn_alloc_info_check(ptr);
   {
     free(ptr);
     if (ptr) {
@@ -1851,9 +2091,13 @@ grn_realloc_default(grn_ctx *ctx, void *ptr, size_t size, const char* file, int
         return NULL;
       }
     }
-    if (!ptr) { GRN_ADD_ALLOC_COUNT(1); }
+    if (!ptr) {
+      GRN_ADD_ALLOC_COUNT(1);
+      grn_alloc_info_add(res);
+    }
   } else {
     if (!ptr) { return NULL; }
+    grn_alloc_info_check(ptr);
     GRN_ADD_ALLOC_COUNT(-1);
 #if defined __FreeBSD__
     free(ptr);
