@@ -793,15 +793,13 @@ grn_pat_add(grn_ctx *ctx, grn_pat *pat, const void *key, uint32_t key_size,
   return r0;
 }
 
-grn_id
-grn_pat_get(grn_ctx *ctx, grn_pat *pat, const void *key, uint32_t key_size, void **value)
+inline static grn_id
+_grn_pat_get(grn_ctx *ctx, grn_pat *pat, const void *key, uint32_t key_size, void **value)
 {
   grn_id r;
   pat_node *rn;
   int c0 = -1, c;
   uint32_t len = key_size * 16;
-  uint8_t keybuf[MAX_FIXED_KEY_SIZE];
-  KEY_ENCODE(pat, keybuf, key, key_size);
   PAT_AT(pat, 0, rn);
   for (r = rn->lr[1]; r;) {
     PAT_AT(pat, r, rn);
@@ -833,6 +831,14 @@ grn_pat_get(grn_ctx *ctx, grn_pat *pat, const void *key, uint32_t key_size, void
   return GRN_ID_NIL;
 }
 
+grn_id
+grn_pat_get(grn_ctx *ctx, grn_pat *pat, const void *key, uint32_t key_size, void **value)
+{
+  uint8_t keybuf[MAX_FIXED_KEY_SIZE];
+  KEY_ENCODE(pat, keybuf, key, key_size);
+  return _grn_pat_get(ctx, pat, key, key_size, value);
+}
+
 grn_id
 grn_pat_nextid(grn_ctx *ctx, grn_pat *pat, const void *key, uint32_t key_size)
 {
@@ -1159,10 +1165,19 @@ const char *
 _grn_pat_key(grn_ctx *ctx, grn_pat *pat, grn_id id, uint32_t *key_size)
 {
   pat_node *node;
+  uint8_t *key;
   PAT_AT(pat, id, node);
-  if (!node) { return NULL; }
-  *key_size = PAT_LEN(node);
-  return (const char *) pat_node_get_key(ctx, pat, node);
+  if (!node) {
+    *key_size = 0;
+    return NULL;
+  }
+  key = pat_node_get_key(ctx, pat, node);
+  if (key) {
+    *key_size = PAT_LEN(node);
+  } else {
+    *key_size = 0;
+  }
+  return (const char *)key;
 }
 
 grn_rc
@@ -1427,7 +1442,7 @@ grn_pat_at(grn_ctx *ctx, grn_pat *pat, grn_id id)
 {
   uint32_t key_size;
   const char *key = _grn_pat_key(ctx, pat, id, &key_size);
-  if (id == grn_pat_get(ctx, pat, key, key_size, NULL)) { return id; }
+  if (key && (id == _grn_pat_get(ctx, pat, key, key_size, NULL))) { return id; }
   return GRN_ID_NIL;
 }
 
@@ -1537,7 +1552,7 @@ grn_pat_cursor_next_by_id(grn_ctx *ctx, grn_pat_cursor *c)
     if (pat->header->n_garbages) {
       uint32_t key_size;
       const void *key = _grn_pat_key(ctx, pat, c->curr_rec, &key_size);
-      if (grn_pat_get(ctx, pat, key, key_size, NULL) != c->curr_rec) {
+      if (_grn_pat_get(ctx, pat, key, key_size, NULL) != c->curr_rec) {
         continue;
       }
     }
@@ -1992,9 +2007,10 @@ grn_pat_cursor_open_by_id(grn_ctx *ctx, grn_pat *pat,
   if (pat->header->n_garbages) {
     while (offset && c->curr_rec != c->tail) {
       uint32_t key_size;
-      const void *key = _grn_pat_key(ctx, pat, c->curr_rec, &key_size);
+      const void *key;
       c->curr_rec += dir;
-      if (grn_pat_get(ctx, pat, key, key_size, NULL) == c->curr_rec) {
+      key = _grn_pat_key(ctx, pat, c->curr_rec, &key_size);
+      if (_grn_pat_get(ctx, pat, key, key_size, NULL) == c->curr_rec) {
         offset--;
       }
     }
