@@ -45,15 +45,6 @@ const char *grn_document_root = NULL;
 #define DEFAULT_DRILLDOWN_LIMIT           10
 #define DEFAULT_DRILLDOWN_OUTPUT_COLUMNS  "_key, _nsubrecs"
 
-#define LAP(msg,num) {\
-  uint64_t et;\
-  grn_timeval tv;\
-  grn_timeval_now(ctx, &tv);\
-  et = (tv.tv_sec - ctx->impl->tv.tv_sec) * GRN_TIME_USEC_PER_SEC\
-    + (tv.tv_usec - ctx->impl->tv.tv_usec);\
-  GRN_LOG(ctx, GRN_LOG_NONE, "%08x|:%012llu %s(%d)", (intptr_t)ctx, et, msg, num);\
-}
-
 grn_rc
 grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
            const char *match_columns, unsigned match_columns_len,
@@ -67,7 +58,8 @@ grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
            const char *drilldown_sortby, unsigned drilldown_sortby_len,
            const char *drilldown_output_columns, unsigned drilldown_output_columns_len,
            int drilldown_offset, int drilldown_limit,
-           const char *cache, unsigned cache_len)
+           const char *cache, unsigned cache_len,
+           const char *match_escalation_threshold, unsigned match_escalation_threshold_len)
 {
   uint32_t nkeys, nhits;
   uint16_t cacheable = 1, taintable = 0;
@@ -80,7 +72,9 @@ grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
   uint32_t cache_key_size = table_len + 1 + match_columns_len + 1 + query_len + 1 +
     filter_len + 1 + scorer_len + 1 + sortby_len + 1 + output_columns_len + 1 +
     drilldown_len + 1 + drilldown_sortby_len + 1 + drilldown_output_columns_len +
+    match_escalation_threshold_len + 1 +
     sizeof(grn_content_type) + sizeof(int) * 4;
+  long long int threshold, original_threshold;
   if (cache_key_size <= GRN_TABLE_MAX_KEY_SIZE) {
     grn_obj *cache;
     char *cp = cache_key;
@@ -104,6 +98,8 @@ grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
     cp += drilldown_sortby_len; *cp++ = '\0';
     memcpy(cp, drilldown_output_columns, drilldown_output_columns_len);
     cp += drilldown_output_columns_len; *cp++ = '\0';
+    memcpy(cp, &match_escalation_threshold, match_escalation_threshold_len);
+    cp += match_escalation_threshold_len; *cp++ = '\0';
     memcpy(cp, &output_type, sizeof(grn_content_type)); cp += sizeof(grn_content_type);
     memcpy(cp, &offset, sizeof(int)); cp += sizeof(int);
     memcpy(cp, &limit, sizeof(int)); cp += sizeof(int);
@@ -112,10 +108,19 @@ grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
     if ((cache = grn_cache_fetch(ctx, cache_key, cache_key_size))) {
       GRN_TEXT_PUT(ctx, outbuf, GRN_TEXT_VALUE(cache), GRN_TEXT_LEN(cache));
       grn_cache_unref(cache_key, cache_key_size);
-      LAP("cache", GRN_TEXT_LEN(cache));
+      LAP(":", "cache(%d)", GRN_TEXT_LEN(cache));
       return ctx->rc;
     }
   }
+  if (match_escalation_threshold_len) {
+    const char *end, *rest;
+    original_threshold = grn_ctx_get_match_escalation_threshold(ctx);
+    end = match_escalation_threshold + match_escalation_threshold_len;
+    threshold = grn_atoll(match_escalation_threshold, end, &rest);
+    if (end == rest) {
+      grn_ctx_set_match_escalation_threshold(ctx, threshold);
+    }
+  }
   if ((table_ = grn_ctx_get(ctx, table, table_len))) {
     // match_columns_ = grn_obj_column(ctx, table_, match_columns, match_columns_len);
     if (query_len || filter_len) {
@@ -168,7 +173,7 @@ grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
       res = table_;
     }
     nhits = res ? grn_table_size(ctx, res) : 0;
-    LAP("select", nhits);
+    LAP(":", "select(%d)", nhits);
     GRN_OUTPUT_ARRAY_OPEN("RESULT", -1);
     if (res) {
       if (scorer && scorer_len) {
@@ -188,7 +193,7 @@ grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
           }
           grn_obj_unlink(ctx, scorer_);
         }
-        LAP("score", nhits);
+        LAP(":", "score(%d)", nhits);
       }
 
       grn_normalize_offset_and_limit(ctx, nhits, &offset, &limit);
@@ -198,7 +203,7 @@ grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
                                        GRN_OBJ_TABLE_NO_KEY, NULL, res))) {
           if ((keys = grn_table_sort_key_from_str(ctx, sortby, sortby_len, res, &nkeys))) {
             grn_table_sort(ctx, res, offset, limit, sorted, keys, nkeys);
-            LAP("sort", limit);
+            LAP(":", "sort(%d)", limit);
             GRN_OBJ_FORMAT_INIT(&format, nhits, 0, limit, offset);
             format.flags =
               GRN_OBJ_FORMAT_WITH_COLUMN_NAMES|
@@ -219,7 +224,7 @@ grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
         GRN_OUTPUT_OBJ(res, &format);
         GRN_OBJ_FORMAT_FIN(ctx, &format);
       }
-      LAP("output", limit);
+      LAP(":", "output(%d)", limit);
       if (!ctx->rc && drilldown_len) {
         uint32_t i, ngkeys;
         grn_table_sort_key *gkeys;
@@ -274,7 +279,7 @@ grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
               }
               grn_obj_unlink(ctx, g.table);
             }
-            LAP("drilldown", nhits);
+            LAP(":", "drilldown(%d)", nhits);
           }
           grn_table_sort_key_close(ctx, gkeys, ngkeys);
         }
@@ -291,6 +296,9 @@ grn_select(grn_ctx *ctx, const char *table, unsigned table_len,
   } else {
     ERR(GRN_INVALID_ARGUMENT, "invalid table name: %.*s", table_len, table);
   }
+  if (match_escalation_threshold_len) {
+    grn_ctx_set_match_escalation_threshold(ctx, original_threshold);
+  }
   /* GRN_LOG(ctx, GRN_LOG_NONE, "%d", ctx->seqno); */
   return ctx->rc;
 }
@@ -334,7 +342,8 @@ proc_select(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
                  GRN_TEXT_VALUE(VAR(10)), GRN_TEXT_LEN(VAR(10)),
                  drilldown_output_columns, drilldown_output_columns_len,
                  drilldown_offset, drilldown_limit,
-                 GRN_TEXT_VALUE(VAR(14)), GRN_TEXT_LEN(VAR(14)))) {
+                 GRN_TEXT_VALUE(VAR(14)), GRN_TEXT_LEN(VAR(14)),
+                 GRN_TEXT_VALUE(VAR(15)), GRN_TEXT_LEN(VAR(15)))) {
   }
   return NULL;
 }
@@ -384,7 +393,7 @@ proc_status(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
   grn_cache_statistics statistics;
   grn_timeval_now(ctx, &now);
   grn_cache_get_statistics(ctx, &statistics);
-  GRN_OUTPUT_MAP_OPEN("RESULT", 12);
+  GRN_OUTPUT_MAP_OPEN("RESULT", 18);
   GRN_OUTPUT_CSTR("alloc_count");
   GRN_OUTPUT_INT32(grn_alloc_count());
   GRN_OUTPUT_CSTR("starttime");
@@ -401,6 +410,12 @@ proc_status(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
   } else {
     GRN_OUTPUT_FLOAT((double)statistics.nhits / (double)statistics.nfetches * 100.0);
   }
+  GRN_OUTPUT_CSTR("command_version");
+  GRN_OUTPUT_INT32(grn_ctx_get_command_version(ctx));
+  GRN_OUTPUT_CSTR("default_command_version");
+  GRN_OUTPUT_INT32(grn_get_default_command_version());
+  GRN_OUTPUT_CSTR("max_command_version");
+  GRN_OUTPUT_INT32(GRN_COMMAND_VERSION_MAX);
   GRN_OUTPUT_MAP_CLOSE();
   return NULL;
 }
@@ -2209,8 +2224,9 @@ grn_db_init_builtin_query(grn_ctx *ctx)
   DEF_VAR(vars[13], "drilldown_offset");
   DEF_VAR(vars[14], "drilldown_limit");
   DEF_VAR(vars[15], "cache");
-  DEF_COMMAND("define_selector", proc_define_selector, 16, vars);
-  DEF_COMMAND("select", proc_select, 15, vars + 1);
+  DEF_VAR(vars[16], "query_escalation_threshold");
+  DEF_COMMAND("define_selector", proc_define_selector, 17, vars);
+  DEF_COMMAND("select", proc_select, 16, vars + 1);
 
   DEF_VAR(vars[0], "values");
   DEF_VAR(vars[1], "table");
