@@ -162,6 +162,18 @@ typedef struct {
 #define GRN_STACK_SIZE 1024
 #define GRN_CTX_N_SEGMENTS 512
 
+#ifdef ENABLE_MEMORY_DEBUG
+typedef struct _grn_alloc_info grn_alloc_info;
+struct _grn_alloc_info
+{
+  void *address;
+  int freed;
+  char alloc_backtrace[4096];
+  char free_backtrace[4096];
+  grn_alloc_info *next;
+};
+#endif
+
 struct _grn_ctx_impl {
   grn_encoding encoding;
 
@@ -179,6 +191,11 @@ struct _grn_ctx_impl {
   grn_strdup_func strdup_func;
 #endif
 
+#ifdef ENABLE_MEMORY_DEBUG
+  /* memory debug portion */
+  grn_alloc_info *alloc_info;
+#endif
+
   /* qe portion */
   grn_obj *stack[GRN_STACK_SIZE];
   uint32_t stack_curr;
@@ -201,6 +218,12 @@ struct _grn_ctx_impl {
   grn_obj names;
   grn_obj levels;
 
+  /* command portion */
+  grn_command_version command_version;
+
+  /* match escalation portion */
+  int64_t match_escalation_threshold;
+
   /* ql portion */
   uint32_t ncells;
   uint32_t seqno;
