@@ -1713,6 +1713,58 @@ grn_ulltob32h(uint64_t i, char *p)
   return p;
 }
 
+grn_rc
+grn_aton(grn_ctx *ctx, const char *p, const char *end, const char **rest,
+         grn_obj *res)
+{
+  if (*p == '+') {
+    p++;
+  }
+
+  switch (*p) {
+  case '-' :
+  case '0' : case '1' : case '2' : case '3' : case '4' :
+  case '5' : case '6' : case '7' : case '8' : case '9' :
+    {
+      int64_t int64;
+      char rest_char;
+      int64 = grn_atoll(p, end, rest);
+      rest_char = **rest;
+      if (end == *rest) {
+        if ((int64_t)INT32_MIN <= int64 && int64 <= (int64_t)INT32_MAX) {
+          grn_obj_reinit(ctx, res, GRN_DB_INT32, 0);
+          GRN_INT32_SET(ctx, res, int64);
+        } else if ((int64_t)INT32_MAX < int64 && int64 <= (int64_t)UINT32_MAX) {
+          grn_obj_reinit(ctx, res, GRN_DB_UINT32, 0);
+          GRN_UINT32_SET(ctx, res, int64);
+        } else {
+          grn_obj_reinit(ctx, res, GRN_DB_INT64, 0);
+          GRN_INT64_SET(ctx, res, int64);
+        }
+      } else if (rest_char == '.' || rest_char == 'e' || rest_char == 'E' ||
+                 (rest_char >= '0' && rest_char <= '9')) {
+        char *rest_float;
+        double d;
+        errno = 0;
+        d = strtod(p, &rest_float);
+        if (!errno && rest_float == end) {
+          grn_obj_reinit(ctx, res, GRN_DB_FLOAT, 0);
+          GRN_FLOAT_SET(ctx, res, d);
+          *rest = rest_float;
+        } else {
+          return GRN_INVALID_ARGUMENT;
+        }
+      }
+    }
+    break;
+  default :
+    return GRN_INVALID_ARGUMENT;
+    break;
+  }
+
+  return GRN_SUCCESS;
+}
+
 int
 grn_str_tok(const char *str, size_t str_len, char delim, const char **tokbuf, int buf_size, const char **rest)
 {
@@ -2661,10 +2713,13 @@ grn_text_otoj(grn_ctx *ctx, grn_obj *bulk, grn_obj *obj, grn_obj_format *format)
         grn_obj *table = grn_ctx_at(ctx, obj->header.domain);
         grn_id id = *((grn_id *)GRN_BULK_HEAD(obj));
         if (table && table->header.type != GRN_TABLE_NO_KEY) {
-          grn_obj *accessor = grn_obj_column(ctx, table, "_key", 4);
-          if (accessor) {
-            grn_obj_get_value(ctx, accessor, id, &buf);
-            grn_obj_unlink(ctx, accessor);
+          /* todo : temporal patch. grn_table_at() is kinda costful... */
+          if (grn_table_at(ctx, table, id)) {
+            grn_obj *accessor = grn_obj_column(ctx, table, "_key", 4);
+            if (accessor) {
+              grn_obj_get_value(ctx, accessor, id, &buf);
+              grn_obj_unlink(ctx, accessor);
+            }
           }
           grn_text_otoj(ctx, bulk, &buf, format);
         } else {
@@ -3170,3 +3225,19 @@ grn_str_url_path_normalize(grn_ctx *ctx, const char *path, size_t path_len,
   }
   *b = '\0';
 }
+
+grn_rc
+grn_text_fgets(grn_ctx *ctx, grn_obj *buf, FILE *fp)
+{
+  size_t len;
+  grn_rc rc = GRN_END_OF_DATA;
+  for (;;) {
+    grn_bulk_reserve(ctx, buf, BUFSIZ);
+    if (!fgets(GRN_BULK_CURR(buf), BUFSIZ, fp)) { break; }
+    if (!(len = strlen(GRN_BULK_CURR(buf)))) { break; }
+    GRN_BULK_INCR_LEN(buf, len);
+    rc = GRN_SUCCESS;
+    if (GRN_BULK_CURR(buf)[-1] == '\n') { break; }
+  }
+  return rc;
+}
