@@ -62,6 +62,7 @@ int64_t grn_atoll(const char *nptr, const char *end, const char **rest);
 grn_rc grn_itoa(int i, char *p, char *end, char **rest);
 grn_rc grn_lltoa(int64_t i, char *p, char *end, char **rest);
 grn_rc grn_ulltoa(uint64_t i, char *p, char *end, char **rest);
+GRN_API grn_rc grn_aton(grn_ctx *ctx, const char *p, const char *end, const char **rest, grn_obj *res);
 const char *grn_enctostr(grn_encoding enc);
 grn_encoding grn_strtoenc(const char *str);
 
@@ -78,8 +79,8 @@ grn_rc grn_substring(grn_ctx *ctx, char **str, char **str_end, int start, int en
 
 void grn_logger_fin(void);
 
-int grn_charlen_(grn_ctx *ctx, const char *str, const char *end, grn_encoding encoding);
-grn_str *grn_str_open_(grn_ctx *ctx, const char *str, unsigned int str_len, int flags, grn_encoding encoding);
+GRN_API int grn_charlen_(grn_ctx *ctx, const char *str, const char *end, grn_encoding encoding);
+GRN_API grn_str *grn_str_open_(grn_ctx *ctx, const char *str, unsigned int str_len, int flags, grn_encoding encoding);
 
 #define GRN_BULK_INCR_LEN(buf,len) {\
   if (GRN_BULK_OUTP(buf)) {\
