@@ -1,3 +1,4 @@
+/* -*- c-basic-offset: 2 -*- */
 /* Copyright(C) 2010 Brazil
 
   This library is free software; you can redistribute it and/or
@@ -14,6 +15,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include "ctx.h"
 #include "db.h"
 #include "ii.h"
 #include "token.h"
@@ -28,15 +30,6 @@
 #define CORRECT  2
 #define SUGGEST  4
 
-#define LAP(msg,num) {\
-  uint64_t et;\
-  grn_timeval tv;\
-  grn_timeval_now(ctx, &tv);\
-  et = (tv.tv_sec - ctx->impl->tv.tv_sec) * GRN_TIME_USEC_PER_SEC\
-    + (tv.tv_usec - ctx->impl->tv.tv_usec);\
-  GRN_LOG(ctx, GRN_LOG_NONE, "%08x|:%012llu %s(%d)", (intptr_t)ctx, et, msg, num);\
-}
-
 static int
 grn_parse_suggest_types(const char *nptr, const char *end)
 {
@@ -106,11 +99,13 @@ cooccur_search(grn_ctx *ctx, grn_obj *table, grn_id id, grn_obj *res, int query_
         ifreq = GRN_INT32_VALUE(&item_freq);
         if (pfreq && ifreq) {
           grn_rset_recinfo *ri;
+          void *value;
           int32_t score = pfreq;
           if (max_score < score) { max_score = score; }
           /* put any formula if desired */
           if (grn_hash_add(ctx, (grn_hash *)res,
-                           &post_id, sizeof(grn_id), (void **)&ri, NULL)) {
+                           &post_id, sizeof(grn_id), &value, NULL)) {
+            ri = value;
             ri->score += score;
           }
         }
@@ -151,7 +146,7 @@ output(grn_ctx *ctx, grn_obj *table, grn_obj *res, grn_id tid,
     }
     if ((keys = grn_table_sort_key_from_str(ctx, sortby_val, sortby_len, res, &nkeys))) {
       grn_table_sort(ctx, res, offset, limit, sorted, keys, nkeys);
-      LAP("sort", limit);
+      LAP(":", "sort(%d)", limit);
       GRN_OBJ_FORMAT_INIT(&format, grn_table_size(ctx, res), 0, limit, offset);
       format.flags =
         GRN_OBJ_FORMAT_WITH_COLUMN_NAMES|
@@ -200,8 +195,10 @@ complete(grn_ctx *ctx, grn_obj *table, grn_obj *col, grn_obj *query, grn_obj *so
                 while ((p = grn_ii_cursor_next(ctx, icur))) {
                   int32_t score;
                   grn_rset_recinfo *ri;
+                  void *value;
                   grn_hash_add(ctx, (grn_hash *)res,
-                               &p->rid, sizeof(grn_id), (void **)&ri, NULL);
+                               &p->rid, sizeof(grn_id), &value, NULL);
+                  ri = value;
                   GRN_BULK_REWIND(&item_freq);
                   GRN_BULK_REWIND(&item_boost);
                   grn_obj_get_value(ctx, items_freq, p->rid, &item_freq);
@@ -229,7 +226,9 @@ complete(grn_ctx *ctx, grn_obj *table, grn_obj *col, grn_obj *query, grn_obj *so
         while ((id = grn_table_cursor_next(ctx, cur))) {
           int32_t score;
           grn_rset_recinfo *ri;
-          grn_hash_add(ctx, (grn_hash *)res, &id, sizeof(grn_id), (void **)&ri, NULL);
+          void *value;
+          grn_hash_add(ctx, (grn_hash *)res, &id, sizeof(grn_id), &value, NULL);
+          ri = value;
           GRN_BULK_REWIND(&item_freq);
           GRN_BULK_REWIND(&item_boost);
           grn_obj_get_value(ctx, items_freq, id, &item_freq);
@@ -262,8 +261,8 @@ correct(grn_ctx *ctx, grn_obj *table, grn_obj *query, grn_obj *sortby,
   if ((res = grn_table_create(ctx, NULL, 0, NULL,
                               GRN_TABLE_HASH_KEY|GRN_OBJ_WITH_SUBREC, table, NULL))) {
     grn_id tid = grn_table_get(ctx, table, TEXT_VALUE_LEN(query));
-    int32_t orig_score, max_score = cooccur_search(ctx, table, tid, res, CORRECT);
-    LAP("cooccur", max_score);
+    int32_t max_score = cooccur_search(ctx, table, tid, res, CORRECT);
+    LAP(":", "cooccur(%d)", max_score);
     if (GRN_TEXT_LEN(query) && max_score < 100) {
       grn_obj *key, *index;
       if ((key = grn_obj_column(ctx, table, CONST_STR_LEN("_key")))) {
@@ -275,7 +274,7 @@ correct(grn_ctx *ctx, grn_obj *table, grn_obj *query, grn_obj *sortby,
           grn_ii_select(ctx, (grn_ii *)index, TEXT_VALUE_LEN(query),
                         (grn_hash *)res, GRN_OP_OR, &optarg);
           grn_obj_unlink(ctx, index);
-          LAP("similar", grn_table_size(ctx, res));
+          LAP(":", "similar(%d)", grn_table_size(ctx, res));
           {
             grn_hash_cursor *hc = grn_hash_cursor_open(ctx, (grn_hash *)res, NULL,
                                                        0, NULL, 0, 0, -1, 0);
@@ -283,9 +282,11 @@ correct(grn_ctx *ctx, grn_obj *table, grn_obj *query, grn_obj *sortby,
               int32_t score;
               grn_id *rp;
               grn_rset_recinfo *ri;
+              void *key, *value;
               while (grn_hash_cursor_next(ctx, hc)) {
-                if (grn_hash_cursor_get_key_value(ctx, hc, (void **)(&rp), NULL,
-                                                  (void **)(&ri))) {
+                if (grn_hash_cursor_get_key_value(ctx, hc, &key, NULL, &value)) {
+                  rp = key;
+                  ri = value;
                   GRN_BULK_REWIND(&item_freq2);
                   GRN_BULK_REWIND(&item_boost);
                   grn_obj_get_value(ctx, items_freq2, *rp, &item_freq2);
@@ -298,7 +299,7 @@ correct(grn_ctx *ctx, grn_obj *table, grn_obj *query, grn_obj *sortby,
               grn_hash_cursor_close(ctx, hc);
             }
           }
-          LAP("filter", grn_table_size(ctx, res));
+          LAP(":", "filter(%d)", grn_table_size(ctx, res));
           {
             /* exec _score -= edit_distance(_key, "query string") for all records */
             grn_obj *var;
@@ -454,8 +455,8 @@ func_suggest_preparer(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *us
           grn_obj_get_value(ctx, events_type, *ep, &pre_type);
           grn_obj_get_value(ctx, events_time, *ep, &pre_time);
           grn_obj_get_value(ctx, events_item, *ep, &pre_item);
-          if (GRN_TIME_VALUE(&pre_time) + 60000000 < post_time) {
-            r = (int)((post_time - GRN_TIME_VALUE(&pre_time))/1000000);
+          if (GRN_TIME_VALUE(&pre_time) + 60 * GRN_TIME_USEC_PER_SEC < post_time) {
+            r = (int)((post_time - GRN_TIME_VALUE(&pre_time))/GRN_TIME_USEC_PER_SEC);
             break;
           }
           key = key_ + GRN_RECORD_VALUE(&pre_item);
@@ -505,6 +506,11 @@ func_suggest_preparer(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *us
   return obj;
 }
 
+/* todo : should be replaced to simple macro call */
+GRN_API grn_rc grn_module_init_suggest(grn_ctx *ctx);
+GRN_API grn_rc grn_module_register_suggest(grn_ctx *ctx);
+GRN_API grn_rc grn_module_fin_suggest(grn_ctx *ctx);
+
 grn_rc
 grn_module_init_suggest(grn_ctx *ctx)
 {
