@@ -41,6 +41,8 @@
 #include "lib/com.h"
 #include "lib/db.h"
 
+GRN_API grn_rc grn_text_fgets(grn_ctx *ctx, grn_obj *buf, FILE *fp);
+
 #ifdef WIN32
 #include <windows.h>
 #include <stddef.h>
@@ -113,7 +115,7 @@ grn_obj *grntest_db = NULL;
 #define BUF_LEN 1024
 #define MAX_PATH_LEN 256
 #define MAX_COMMAND_LEN 500000
-#define LOGBUF_LEN 200000
+#define LOGBUF_LEN 5000000
 
 #define J_DO_LOCAL  1  /* do_local */
 #define J_DO_GQTP   2  /* do_gqtp */
@@ -389,10 +391,10 @@ report_load_command(grn_ctx *ctx, char *ret, int task_id, long long int start_ti
   start = start_time - GRN_TIME_VALUE(&grntest_starttime);
   end = GRN_TIME_VALUE(end_time) - GRN_TIME_VALUE(&grntest_starttime);
   if (grntest_outtype == OUT_TSV) {
-    fprintf(grntest_logfp, "report\t%d\tload\t%lld\t%lld\t%s\n",
+    fprintf(grntest_logfp, "report\t%d\tload\t%" GRN_FMT_LLD "\t%" GRN_FMT_LLD "\t%s\n",
             task_id,  start, end, rettmp);
   } else {
-    fprintf(grntest_logfp, "[%d, \"load\", %lld, %lld, %s],\n",
+    fprintf(grntest_logfp, "[%d, \"load\", %" GRN_FMT_LLD ", %" GRN_FMT_LLD ", %s],\n",
             task_id,  start, end, rettmp);
   }
   fflush(grntest_logfp);
@@ -432,10 +434,10 @@ report_command(grn_ctx *ctx, char *command, char *ret, int task_id,
   clen = strlen(command);
   escape_command(command, clen, command_escaped, clen * 2);
   if (grntest_outtype == OUT_TSV) {
-    fprintf(grntest_logfp, "report\t%d\t%s\t%lld\t%lld\t%s\n",
+    fprintf(grntest_logfp, "report\t%d\t%s\t%" GRN_FMT_LLD "\t%" GRN_FMT_LLD "\t%s\n",
             task_id, command_escaped, start, end, rettmp);
   } else {
-    fprintf(grntest_logfp, "[%d, \"%s\", %lld, %lld, %s],\n",
+    fprintf(grntest_logfp, "[%d, \"%s\", %" GRN_FMT_LLD ", %" GRN_FMT_LLD ", %s],\n",
             task_id, command_escaped, start, end, rettmp);
   }
   fflush(grntest_logfp);
@@ -458,10 +460,10 @@ output_result_final(grn_ctx *ctx, int qnum)
   sec = self / (double)1000000;
   qps = (double)qnum / sec;
   if (grntest_outtype == OUT_TSV) {
-    fprintf(grntest_logfp, "total\t%lld\t%f\t%d\n", latency, qps, qnum);
+    fprintf(grntest_logfp, "total\t%" GRN_FMT_LLD "\t%f\t%d\n", latency, qps, qnum);
   } else {
     fprintf(grntest_logfp,
-           "{\"total\": %lld, \"qps\": %f, \"queries\": %d}]\n", latency, qps, qnum);
+           "{\"total\": %" GRN_FMT_LLD ", \"qps\": %f, \"queries\": %d}]\n", latency, qps, qnum);
   }
   grn_obj_close(ctx, &end_time);
   return 0;
@@ -499,6 +501,8 @@ shutdown_server(void)
   return 0;
 }
 
+/* #define ENABLE_ERROR_REPORT 1 */
+#ifdef ENABLE_ERROR_REPORT
 static
 int
 error_command(grn_ctx *ctx, char *command, int task_id)
@@ -508,6 +512,7 @@ error_command(grn_ctx *ctx, char *command, int task_id)
   error_exit_in_thread(1);
   return 0;
 }
+#endif
 
 static
 int
@@ -899,7 +904,8 @@ do_load_command(grn_ctx *ctx, char *command, int type, int task_id,
           error_exit_in_thread(55);
         }
         if (logbuf[LOGBUF_LEN-2] != '\0') {
-          fprintf(stderr, "too long line input-log\n");
+          fprintf(stderr, "too long line input-log: <%s>: <%.*s>\n",
+                  command, LOGBUF_LEN - 2, logbuf);
           error_exit_in_thread(56);
         }
         loglen = strlen(logbuf);
@@ -984,7 +990,8 @@ do_command(grn_ctx *ctx, char *command, int type, int task_id)
           error_exit_in_thread(55);
         }
         if (logbuf[LOGBUF_LEN-2] != '\0') {
-          fprintf(stderr, "too long line input-log\n");
+          fprintf(stderr, "too long line input-log: <%s>: <%.*s>\n",
+                  command, LOGBUF_LEN - 2, logbuf);
           error_exit_in_thread(56);
         }
         loglen = strlen(logbuf);
@@ -1003,9 +1010,9 @@ do_command(grn_ctx *ctx, char *command, int type, int task_id)
       grn_obj_close(ctx, &end_time);
       break;
     } else {
-/*
+#ifdef ENABLE_ERROR_REPORT
       error_command(ctx, command, task_id);
-*/
+#endif
     }
   } while ((flags & GRN_CTX_MORE));
 
@@ -1154,14 +1161,14 @@ worker_sub(intptr_t task_id)
     grntest_jobdone++;
     if (grntest_outtype == OUT_TSV) {
       sprintf(tmpbuf,
-              "job\t%s\t%lld\t%lld\t%f\t%lld\t%lld\t%d\n",
+              "job\t%s\t%" GRN_FMT_LLD "\t%" GRN_FMT_LLD "\t%f\t%" GRN_FMT_LLD "\t%" GRN_FMT_LLD "\t%d\n",
               grntest_job[grntest_task[task_id].job_id].jobname, latency, self, qps,
               grntest_job[grntest_task[task_id].job_id].min,
               grntest_job[grntest_task[task_id].job_id].max,
               grntest_job[grntest_task[task_id].job_id].qnum);
     } else {
       sprintf(tmpbuf,
-              "{\"job\": \"%s\", \"latency\": %lld, \"self\": %lld, \"qps\": %f, \"min\": %lld, \"max\": %lld, \"queries\": %d}",
+              "{\"job\": \"%s\", \"latency\": %" GRN_FMT_LLD ", \"self\": %" GRN_FMT_LLD ", \"qps\": %f, \"min\": %" GRN_FMT_LLD ", \"max\": %" GRN_FMT_LLD ", \"queries\": %d}",
               grntest_job[grntest_task[task_id].job_id].jobname, latency, self, qps,
               grntest_job[grntest_task[task_id].job_id].min,
               grntest_job[grntest_task[task_id].job_id].max,
@@ -1503,9 +1510,9 @@ get_sysinfo(const char *path, char *result, int olen)
   }
 
   if (grntest_outtype == OUT_TSV) {
-    sprintf(tmpbuf, "%ldKBytes\n", vfsbuf.f_blocks * 4);
+    sprintf(tmpbuf, "%luKBytes\n", vfsbuf.f_blocks * 4);
   } else {
-    sprintf(tmpbuf, "  \"HDD\": \"%ldKBytes\",\n", vfsbuf.f_blocks * 4);
+    sprintf(tmpbuf, "  \"HDD\": \"%luKBytes\",\n", vfsbuf.f_blocks * 4);
   }
   strcat(result, tmpbuf);
 
@@ -3085,10 +3092,10 @@ main(int argc, char **argv)
   get_date(grntest_date, &sec);
 
   if (outdir) {
-    sprintf(log, "%s/%s-%s-%lld-%s.log", outdir, grntest_scriptname,
+    sprintf(log, "%s/%s-%s-%" GRN_FMT_LLD "-%s.log", outdir, grntest_scriptname,
             grntest_username, GRN_TIME_VALUE(&grntest_starttime), grn_get_version());
   } else {
-    sprintf(log, "%s-%s-%lld-%s.log", grntest_scriptname,
+    sprintf(log, "%s-%s-%" GRN_FMT_LLD "-%s.log", grntest_scriptname,
             grntest_username, GRN_TIME_VALUE(&grntest_starttime), grn_get_version());
   }
 
