@@ -15,6 +15,9 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#ifdef WIN32
+#define GROONGA_MAIN
+#endif /* WIN32 */
 #include "lib/groonga_in.h"
 
 #include "lib/com.h"
@@ -106,6 +109,10 @@ usage(FILE *output)
           "  --config-path <path>:             specify config file path\n"
           "  --cache-limit <limit>:            specify the max number of cache data\n"
           "  --file <path>:                    read commands from specified file\n"
+          "  --default-command-version <version>:\n"
+          "                                    specify default command version\n"
+          "  --default-match-escalation-threshold <threshold>:\n"
+          "                                    specify default match escalation threshold\n"
           "\n"
           "dest: <db pathname> [<command>] or <dest hostname>\n"
           "  <db pathname> [<command>]: when standalone/server mode\n"
@@ -130,9 +137,8 @@ show_version(void)
 #endif
   printf("%s", GROONGA_DEFAULT_ENCODING);
 
-#ifdef GROONGA_DEFAULT_QUERY_ESCALATION_THRESHOLD
-  printf(",query-cache=%ds", GROONGA_DEFAULT_QUERY_ESCALATION_THRESHOLD);
-#endif
+  printf(",match-escalation-threshold=%" GRN_FMT_LLD,
+         grn_get_default_match_escalation_threshold());
 
 #ifndef NO_NFKC
   printf(",nfkc");
@@ -481,11 +487,11 @@ print_return_code(grn_ctx *ctx, grn_rc rc, grn_obj *head, grn_obj *body, grn_obj
       grn_timeval tv;
       grn_timeval_now(ctx, &tv);
       dv = ctx->impl->tv.tv_sec;
-      dv += ctx->impl->tv.tv_usec / 1000000.0;
+      dv += ctx->impl->tv.tv_nsec / GRN_TIME_NSEC_PER_SEC_F;
       GRN_TEXT_PUTC(ctx, head, ',');
       grn_text_ftoa(ctx, head, dv);
       dv = (tv.tv_sec - ctx->impl->tv.tv_sec);
-      dv += (tv.tv_usec - ctx->impl->tv.tv_usec) / 1000000.0;
+      dv += (tv.tv_nsec - ctx->impl->tv.tv_nsec) / GRN_TIME_NSEC_PER_SEC_F;
       GRN_TEXT_PUTC(ctx, head, ',');
       grn_text_ftoa(ctx, head, dv);
     }
@@ -515,10 +521,10 @@ print_return_code(grn_ctx *ctx, grn_rc rc, grn_obj *head, grn_obj *body, grn_obj
       grn_timeval tv;
       grn_timeval_now(ctx, &tv);
       dv = ctx->impl->tv.tv_sec;
-      dv += ctx->impl->tv.tv_usec / 1000000.0;
+      dv += ctx->impl->tv.tv_nsec / GRN_TIME_NSEC_PER_SEC_F;
       grn_text_ftoa(ctx, head, dv);
       dv = (tv.tv_sec - ctx->impl->tv.tv_sec);
-      dv += (tv.tv_usec - ctx->impl->tv.tv_usec) / 1000000.0;
+      dv += (tv.tv_nsec - ctx->impl->tv.tv_nsec) / GRN_TIME_NSEC_PER_SEC_F;
       GRN_TEXT_PUTC(ctx, head, '\t');
       grn_text_ftoa(ctx, head, dv);
     }
@@ -564,10 +570,10 @@ print_return_code(grn_ctx *ctx, grn_rc rc, grn_obj *head, grn_obj *body, grn_obj
           grn_timeval tv;
           grn_timeval_now(ctx, &tv);
           dv = ctx->impl->tv.tv_sec;
-          dv += ctx->impl->tv.tv_usec / 1000000.0;
+          dv += ctx->impl->tv.tv_nsec / GRN_TIME_NSEC_PER_SEC_F;
           grn_text_ftoa(ctx, head, dv);
           dv = (tv.tv_sec - ctx->impl->tv.tv_sec);
-          dv += (tv.tv_usec - ctx->impl->tv.tv_usec) / 1000000.0;
+          dv += (tv.tv_nsec - ctx->impl->tv.tv_nsec) / GRN_TIME_NSEC_PER_SEC_F;
           GRN_TEXT_PUTS(ctx, head, "\" ELAPSED=\"");
           grn_text_ftoa(ctx, head, dv);
           GRN_TEXT_PUTS(ctx, head, "\">");
@@ -662,18 +668,6 @@ do_alone(int argc, char **argv)
   return rc;
 }
 
-#define BATCHMODE(ctx) do {\
-  int flags;\
-  unsigned int str_len;\
-  char *str, *query = "(batchmode #t)";\
-  grn_ctx_send(ctx, query, strlen(query), 0);\
-  do {\
-    if (grn_ctx_recv(ctx, &str, &str_len, &flags)) {\
-      fprintf(stderr, "grn_ctx_recv failed\n");\
-    }\
-  } while ((flags & GRN_CTX_MORE));\
-} while (0)
-
 static int
 c_output(grn_ctx *ctx)
 {
@@ -724,7 +718,6 @@ g_client(int argc, char **argv)
       if (!rc) {
         char *buf = GRN_TEXT_VALUE(&text);
         int   len;
-        if (batchmode) { BATCHMODE(ctx); }
         while ((len = prompt(ctx, buf))) {
           uint32_t size = len - 1;
           grn_ctx_send(ctx, buf, size, 0);
@@ -764,9 +757,6 @@ h_output(grn_ctx *ctx, int flags, void *arg)
   grn_rc expr_rc = ctx->rc;
   ht_context *hc = (ht_context *)arg;
   grn_sock fd = hc->msg->u.fd;
-  grn_obj *expr = ctx->impl->curr_expr;
-  grn_obj *jsonp_func = grn_expr_get_var(ctx, expr, JSON_CALLBACK_PARAM,
-                                         strlen(JSON_CALLBACK_PARAM));
   grn_obj *body = &hc->body;
   const char *mime_type = ctx->impl->mime_type;
   grn_obj head, foot, *outbuf = ctx->impl->outbuf;
@@ -774,6 +764,12 @@ h_output(grn_ctx *ctx, int flags, void *arg)
   GRN_TEXT_INIT(&head, 0);
   GRN_TEXT_INIT(&foot, 0);
   if (!expr_rc) {
+    grn_obj *expr = ctx->impl->curr_expr;
+    grn_obj *jsonp_func = NULL;
+    if (expr) {
+      expr = grn_expr_get_var(ctx, expr, JSON_CALLBACK_PARAM,
+                              strlen(JSON_CALLBACK_PARAM));
+    }
     if (jsonp_func && GRN_TEXT_LEN(jsonp_func)) {
       GRN_TEXT_PUT(ctx, &head, GRN_TEXT_VALUE(jsonp_func), GRN_TEXT_LEN(jsonp_func));
       GRN_TEXT_PUTC(ctx, &head, '(');
@@ -1479,7 +1475,10 @@ h_worker(void *arg)
     nfthreads++;
     while (!(msg = (grn_obj *)grn_com_queue_deque(&grn_gctx, &ctx_new))) {
       COND_WAIT(q_cond, q_mutex);
-      if (grn_gctx.stat == GRN_CTX_QUIT) { goto exit; }
+      if (grn_gctx.stat == GRN_CTX_QUIT) {
+        nfthreads--;
+        goto exit;
+      }
     }
     nfthreads--;
     MUTEX_UNLOCK(q_mutex);
@@ -1632,7 +1631,10 @@ g_worker(void *arg)
     nfthreads++;
     while (!(edge = (grn_edge *)grn_com_queue_deque(&grn_gctx, &ctx_new))) {
       COND_WAIT(q_cond, q_mutex);
-      if (grn_gctx.stat == GRN_CTX_QUIT) { goto exit; }
+      if (grn_gctx.stat == GRN_CTX_QUIT) {
+        nfthreads--;
+        goto exit;
+      }
     }
     ctx = &edge->ctx;
     nfthreads--;
@@ -2063,7 +2065,8 @@ main(int argc, char **argv)
   const char *portstr = NULL, *encstr = NULL,
     *max_nfthreadsstr = NULL, *loglevel = NULL,
     *listen_addressstr = NULL, *hostnamestr = NULL, *protocol = NULL,
-    *cache_limitstr = NULL, *admin_html_path = NULL;
+    *cache_limitstr = NULL, *admin_html_path = NULL, *command_versionstr = NULL,
+    *match_escalation_thresholdstr = NULL;
   const char *config_path = NULL;
   const char *input_path = NULL;
   int r, i, mode = mode_alone;
@@ -2091,6 +2094,8 @@ main(int argc, char **argv)
     {'\0', "cache-limit", NULL, 0, getopt_op_none},
     {'\0', "file", NULL, 0, getopt_op_none},
     {'\0', "document-root", NULL, 0, getopt_op_none},
+    {'\0', "default-command-version", NULL, 0, getopt_op_none},
+    {'\0', "default-match-escalation-threshold", NULL, 0, getopt_op_none},
     {'\0', NULL, NULL, 0, 0}
   };
   opts[0].arg = &portstr;
@@ -2108,6 +2113,8 @@ main(int argc, char **argv)
   opts[20].arg = &cache_limitstr;
   opts[21].arg = &input_path;
   opts[22].arg = &grn_document_root;
+  opts[23].arg = &command_versionstr;
+  opts[24].arg = &match_escalation_thresholdstr;
   if (!(default_max_nfthreads = get_core_number())) {
     default_max_nfthreads = DEFAULT_MAX_NFTHREADS;
   }
@@ -2244,6 +2251,21 @@ main(int argc, char **argv)
 #endif
   if (grn_init()) { return -1; }
   grn_set_default_encoding(enc);
+  if (command_versionstr) {
+    grn_set_default_command_version(atoi(command_versionstr));
+  }
+  if (match_escalation_thresholdstr) {
+    int64_t threshold;
+    const char *end, *rest;
+    end = match_escalation_thresholdstr + strlen(match_escalation_thresholdstr);
+    threshold = grn_atoll(match_escalation_thresholdstr, end, &rest);
+    if (end != rest) {
+      fprintf(stderr, "invalid default match escalation threshold: <%s>\n",
+              match_escalation_thresholdstr);
+      return EXIT_FAILURE;
+    }
+    grn_set_default_match_escalation_threshold(threshold);
+  }
   if (loglevel) { SET_LOGLEVEL(atoi(loglevel)); }
   grn_set_segv_handler();
   grn_set_int_handler();
