@@ -44,8 +44,7 @@ void test_match_without_index(void);
 void
 cut_startup(void)
 {
-  tmp_directory = g_build_filename(grn_test_get_base_dir(),
-                                   "tmp",
+  tmp_directory = g_build_filename(grn_test_get_tmp_dir(),
                                    "test-table-select",
                                    NULL);
 }
