@@ -24,6 +24,7 @@
 void test_in_circle(void);
 void test_in_rectangle_long_longitude(void);
 void test_in_rectangle_long_latitude(void);
+void test_in_rectangle_over_border(void);
 void test_sort(void);
 void test_filter_by_tag_and_sort_by_distance_from_tokyo_tocho(void);
 void test_in_circle_and_tag(void);
@@ -189,6 +190,20 @@ test_in_rectangle_long_latitude(void)
         grn_test_location_string(budoukan_latitude, budoukan_longitude))));
 }
 
+void
+test_in_rectangle_over_border(void)
+{
+  grn_test_assert_send_command_error(
+    context,
+    GRN_FUNCTION_NOT_IMPLEMENTED,
+    "geo_in_rectangle() with negative coordinate is not implemented.",
+    "select Shops "
+    "--sortby '+name' "
+    "--output_columns 'name, location' "
+    "--filter 'geo_in_rectangle(location, "
+    "\"35.73360x-139.7394\", \"-35.62614x139.7714\")'");
+}
+
 void
 test_sort(void)
 {
